const { ccclass, property } = cc._decorator;

@ccclass
export default class Game extends cc.Component {

    @property(cc.RichText)
    describe: cc.RichText = null;

    storyArr: string[] = [
        '每次我不知道说什么的时候我就不知道说什么。',
        '同一个小区里的邻居，他们家的孩子六年级就开始上六年级的课了。我家的孩子三年级，还在上三年级的课',
        '经过我细密的观察，我发现我出生的哪一天竟然是我的生日',
        '如果你登上了珠穆朗玛峰山顶，那么你就具备了登上珠穆朗玛峰山顶的能力',
        '如果你愿意多花一点事件读完，那么你就会发现你多花了一点时间',
    ];
    storyIdx: number = 0;

    protected onLoad(): void {
        this.describe.string = '';
    }

    onBtnPlayTypewriterEffect() {
        let str = this.storyArr[this.storyIdx];
        this.playTypewriterAnim(this.describe, str, 1000, () => {
            this.storyIdx++;
            if (this.storyIdx >= this.storyArr.length) this.storyIdx = 0;
            console.log('finish~~~');
        });
    }


    /**
     * 富文本打字机
     * @param {cc.RichText} animLab 富文本组件
     * @param {string} storyStr 文本
     * @param {number} speed 打字机速度
     * @param {Function} callback 打字完成回调
     */
    playTypewriterAnim(animLab: cc.RichText, storyStr: string, speed: number = 1000, callback?: Function) {
        animLab.string = '';
        animLab.node.removeAllChildren();
        animLab.string = storyStr;

        //富文本RichText组件是分为多个文本Label组件实现的，这一点在私有属性_labelSegments中可以发现
        let _labelSegments = animLab._labelSegments;
        let delay = 0;
        let tweenAnim = [];
        for (let i = 0; i < _labelSegments.length; i++) {
            const element = _labelSegments[i];
            console.log(element.getComponent(cc.Label).string + ' x:' + element.x + ' y:' + element.y);
            const maskNode = new cc.Node();
            maskNode.name = `mask${i}`;
            maskNode.addComponent(cc.Mask);
            maskNode.anchorX = element.anchorX;
            maskNode.anchorY = element.anchorY;
            maskNode.x = element.x + animLab.node.width / 2;
            maskNode.y = element.y - animLab.node.height / 2;
            maskNode.setContentSize(element.getContentSize());
            element.parent = maskNode;
            element.x = 0;
            element.y = 0;
            element.anchorX = 0.5;
            element.anchorY = 0.5;
            animLab.node.addChild(maskNode);
            let width = maskNode.width;
            let dur = width / speed;
            tweenAnim.push({ node: maskNode, delay: delay, dur: dur, width: width })
            delay += dur;
        }
        animLab.string = '';
        let parentY = animLab.node.y;
        animLab.node.y = 2600;
        this.scheduleOnce(() => {
            for (let i = 0; i < tweenAnim.length; i++) {
                const element = tweenAnim[i];
                element.node.width = 0;
                if (i < tweenAnim.length - 1) {
                    cc.tween(element.node).delay(element.delay).to(element.dur, { width: element.width }).start();
                } else {
                    cc.tween(element.node).delay(element.delay).to(element.dur, { width: element.width }).call(() => {
                        if (callback) callback();
                    }).start();
                }
            }
            animLab.node.y = parentY;
        }, 0);
    }
}
